var modelfactory = require('.'),
    model = modelfactory.model,
    Schema = modelfactory.Schema,
    Mixed = Schema.Types.Mixed,
    ObjectId = Schema.Types.ObjectId;

// global plugin
modelfactory.plugin(function(schema) {
  schema.static('save', function() {});
});

//Sample Project sche
var ProjectSchema = new Schema({
  name: {type: String},
  category: {type: String, enum: ['Web', 'Mobile']}
});

ProjectSchema.path('category').set(function(v) {
  return v[0].toUpperCase() + v.slice(1);
});

ProjectSchema.plugin(function(schema) {
  schema.static('foo', function() {});
});

// Sample User Schema
var UserSchema = new Schema({
    email: {
      type: String,
      required: true,
      match: /^[a-zA-Z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$/
    },

    sex: {
      type: String,
      enum: ['unkown', 'female', 'male']
    },

    creditcard: String,

    // test firefox watch
    watch: {
      stuff: Boolean
    },

    age: {
      type: Number,
      min: 7,
      max: 77
    },

    date: {
      type: Date,
    },

    createdAt: {
      type: Date,
      default: Date.now
    },

    defaultStuff: {
      type: String,
      default: 'stuff'
    },

    someRef: {
      type: ObjectId
    },

    project: {type: ProjectSchema, required: true},

    projects: [ProjectSchema],

    projects2: {
      type: [ProjectSchema],
      default: []
    },

    tags: [{type: String, enum: ['js', 'html']}],

    keywords: {
      type: [String],
      default: ['one', 'two'],
      max: 3
    },

    stuffs: [
      {
        foo: String,
        bar: Boolean
      }
    ],

    whatever: { type: Mixed },
    whatevers: Array,

    name: {
      first: {
        type: String,
        required: true
      },

      last: {
        type: String,
        required: true
      }
    },

    one: {
      two: {
        tree: {
          type: Boolean
        }
      }
    }
  });

UserSchema.store.index('email');

UserSchema.path('name.last').set(function(v) {
  return v[0].toUpperCase() + v.slice(1);
});

UserSchema.path('creditcard').get(function(v) {
  if (!v) return v;
  return v.slice(0, 3) + '**********';
});

UserSchema.method('hello', function() {
  return 'hello ' + this.name.first;
});

UserSchema.static('version', function() {
  return '0.1';
});

UserSchema.virtual('boo')
  .get(function() { return this._boo; })
  .set(function(v) { this._boo = v; });

UserSchema.virtual('name.full').get(function() {
  return this.name.first + ' ' + this.name.last;
});

UserSchema.virtual('name.full').set(function(v) {
  var split = v.split(' ');
  this.name.first = split.shift();
  this.name.last = split.join(' ');
});

var User = model('User', UserSchema),
    Project = model(ProjectSchema),
    user,
    emit;

user = new User({
  email: 'john@gmail.com',
  name: {first: 'john', last: 'mcenroe'},
  age: 44,
  creditcard: '123-456-789',
  sex: 'male',
  project: {name: 'current project', category: 'web'},
  projects: [
    {name: 'project1', category: 'web'},
    {name: 'project2', category: 'mobile'}
  ],
  keywords: ['foo', 'bar'],
  whatever: { foo: { bar: 'bar' } },
  whatevers: [{foo: 'foo'}, {bar: 'bar'}],
  tags: ['js'],
  date: new Date(),
  one: {two: {tree: true}}
});



